@@ -86,7 +86,8 @@ SEXP cpp_which_(SEXP x, bool invert){
 SEXP cpp_which_val(SEXP x, SEXP value, bool invert){
   int n_protections = 0;
   R_xlen_t n = Rf_xlength(x);
-  if (cpp_vec_length(value) != 1){
+  bool is_long = (n > integer_max_);
+  if (Rf_length(value) != 1){
     Rf_error("value must be a vector of length 1");
   }
   SEXP val_is_na = Rf_protect(cpp_is_na(value));
@@ -99,6 +100,18 @@ SEXP cpp_which_val(SEXP x, SEXP value, bool invert){
       return cpp_which_na(x);
     }
   }
+#define WHICH_VAL(_val_)                                           \
+  if (invert){                                                     \
+    while (whichi < out_size){                                     \
+      p_out[whichi] = i + 1;                                       \
+      whichi += (p_x[i++] != _val_);                               \
+    }                                                              \
+  } else {                                                         \
+    while (whichi < out_size){                                     \
+      p_out[whichi] = i + 1;                                       \
+      whichi += (p_x[i++] == _val_);                               \
+    }                                                              \
+  }
   R_xlen_t n_vals = scalar_count(x, value, false);
   R_xlen_t out_size = invert ? n - n_vals : n_vals;
   R_xlen_t whichi = 0;
@@ -106,68 +119,53 @@ SEXP cpp_which_val(SEXP x, SEXP value, bool invert){
   switch ( TYPEOF(x) ){
   case LGLSXP:
   case INTSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
+    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
     ++n_protections;
-    int *p_out = INTEGER(out);
     Rf_protect(value = Rf_coerceVector(value, INTSXP));
     ++n_protections;
     int val = Rf_asInteger(value);
     int *p_x = INTEGER(x);
-    if (invert){
-      while (whichi < out_size){
-        p_out[whichi] = i + 1;
-        whichi += !(p_x[i++] == val);
-      }
+    if (is_long){
+      double *p_out = REAL(out);
+      WHICH_VAL(val);
     } else {
-      while (whichi < out_size){
-        p_out[whichi] = i + 1;
-        whichi += (p_x[i++] == val);
-      }
+      int *p_out = INTEGER(out);
+      WHICH_VAL(val);
     }
     Rf_unprotect(n_protections);
     return out;
   }
   case REALSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
+    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
     ++n_protections;
-    int *p_out = INTEGER(out);
     Rf_protect(value = Rf_coerceVector(value, REALSXP));
     ++n_protections;
     double val = Rf_asReal(value);
     double *p_x = REAL(x);
-    if (invert){
-      while (whichi < out_size){
-        p_out[whichi] = i + 1;
-        whichi += !(p_x[i++] == val);
-      }
+    if (is_long){
+      double *p_out = REAL(out);
+      WHICH_VAL(val);
     } else {
-      while (whichi < out_size){
-        p_out[whichi] = i + 1;
-        whichi += (p_x[i++] == val);
-      }
+      int *p_out = INTEGER(out);
+      WHICH_VAL(val);
     }
     Rf_unprotect(n_protections);
     return out;
   }
   case STRSXP: {
-    SEXP out = Rf_protect(Rf_allocVector(INTSXP, out_size));
+    SEXP out = Rf_protect(Rf_allocVector(is_long ? REALSXP : INTSXP, out_size));
     ++n_protections;
-    int *p_out = INTEGER(out);
     Rf_protect(value = Rf_coerceVector(value, STRSXP));
     ++n_protections;
     SEXP val = Rf_protect(Rf_asChar(value));
     ++n_protections;
     SEXP *p_x = STRING_PTR(x);
-    if (invert){
-      while (whichi < out_size){
-        p_out[whichi] = i + 1;
-        whichi += !(p_x[i++] == val);
-      }
+    if (is_long){
+      double *p_out = REAL(out);
+      WHICH_VAL(val);
     } else {
-      while (whichi < out_size){
-        p_out[whichi] = i + 1;
-        whichi += (p_x[i++] == val);
-      }
+      int *p_out = INTEGER(out);
+      WHICH_VAL(val);
     }
     Rf_unprotect(n_protections);
     return out;
